#!/bin/sh
#
# create_msqt_schema.sh  -  Create a new schema for an existing msqt database
#
#
# written 2006 by Joffrey Fitz fitzj@informatik.uni-tuebingen.de
#
# This script is part of the MSQT package written by
# Norman Warthmann and Joffrey Fitz
# http://msqt.weigelworld.org
#
#
# Requirements: 
#  - BioPerl 1.5
#  - seq
#  - GNU sed
#  - get_allelels_batch.pl
#  - get_allelels_batch_pos_name.pl
#  - produce_target_sequence_table.pl
#  - produce_target_sequence_table_pos_name.pl
#
#
#  TODO:
#  - Rewrite sed and seq in perl
#  - Check return values of psql
#  - Check $PAGER variable, not set in sh on OS X
#  - Check for dirs and .files in data directory. Should be omitted
#
#
# Installing BioPerl 1.5 from CVS
#
# cvs -d :pserver:cvs@code.open-bio.org:/home/repository/bioperl login
# # password: cvs
# cvs -d :pserver:cvs@code.open-bio.org:/home/repository/bioperl checkout bioperl-live 
# # Setting PERL5LIB enviroment variable
# export PERL5LIB="$PERL5LIB:/home/jfitz/mpi/msqt_install/bioperl-1.5/bioperl-live"
#




CONFIGFILE="../config/msqt.conf-new"


#######################################################################################
# Do not edit anything below this line
#
DBUSER="msqt"
WORKDIR=".work" 		# for configured sql scripts
RESDIR=".results" 	# storing parsed fasta files results
SQLDIR="sql"
SQLFILES=`ls $SQLDIR`
LOGDIR="log"
LOGFILE="create_msqt_schema.log"



#######################################################################################
# Function definitions

read_config_file() {
	unset DBHOST DBPORT DBNAME SCHEMANAME PATHTODATA CHRDIRPREFIX
	if [ -r ${CONFIGFILE} ]; then
		. ${CONFIGFILE}
	else
		fail "Missing or unreadable ${CONFIGFILE}"
	fi
	
	if [ -z "$DBHOST" ] || [ -z "$DBPORT" ] || [ -z "$DBNAME" ] || [ -z "$SCHEMANAME" ] || [ -z "$PATHTODATA" ]; then
		echo "!! Missing configuration DBHOST DBPORT DBNAME SCHEMANAME PATHTODATA and/or CHRDIRPREFIX information"
		echo "!! Check your config file (${CONFIGFILE})."
		exit 1;
	fi
}

test_requirements() {
	
	echo "Testing dependencies"
	echo
	# tools and paths
	echo -n "Checking for perl ... "
	type perl > /dev/null 2>&1
	if [ $? -eq 0 ] ; then _perl=`which perl 2> /dev/null`; echo "ok (${_perl})"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for psql ... "
	type psql > /dev/null 2>&1
	if [ $? -eq 0 ] ; then _psql=`which psql 2> /dev/null`; echo "ok (${_psql})"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for cat ... "
	type cat > /dev/null 2>&1
	if [ $? -eq 0 ] ; then _cat=`which cat 2> /dev/null`; echo "ok (${_cat})"
	else __internal_error=1; echo "no"; fi

	echo -n "Checking for less ... "
	type less > /dev/null 2>&1
	if [ $? -eq 0 ] ; then PAGER=`which less 2> /dev/null`; echo "ok ($PAGER)"
	else __internal_error=1; echo "no"; fi
	
	
	# Perl modules

	# Modules needed by get_alleles_batch* and produce_target_sequence*
	echo -n "Checking for Bio::AlignIO ... "
	perl -MBio::AlignIO -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for Bio::PopGen::Utilities ... "
	perl -MBio::PopGen::Utilities -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for Bio::PopGen::PopulationI ... "
	perl -MBio::PopGen::PopulationI -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for Bio::PopGen::MarkerI ... "
	perl -MBio::PopGen::MarkerI -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for File::Basename ... "
	perl -MFile::Basename -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi

	# Modules required by cgi programs
	echo -n "Checking for CGI ... "
	perl -MCGI -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for DBI ... "
	perl -MDBI -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for DBD::Pg ... "
	perl -MDBD::Pg -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for Switch ... "
	perl -MSwitch -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	# Modules required by Alivie
	echo -n "Checking for LWP::Simple ... "
	perl -MLWP::Simple -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for URI ... "
	perl -MURI -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	echo -n "Checking for XML::Simple ... "
	perl -MXML::Simple -e "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1; echo "no"; fi
	
	# check status
	echo
	if [ ${__internal_error} ]; then
		echo "!! Missing Requirements. Exiting."; exit 1
	else
		echo "All dependencies resolved."
	fi

	# reste error flag
	__internal_error=""
}

test_db() {
	
	# Test database connection
	echo
	echo "Testing configuration"
	echo
	echo -n "Testing database connection ... "
	psql -U $DBUSER -d $DBNAME -h $DBHOST -p $DBPORT -c "" 2> /dev/null
	if [ $? -eq 0 ] ; then echo "ok"
	else __internal_error=1
		echo "failed"
		echo "!! Can not connect to database. Please check your configuration."
		echo
		echo "Used parameters: "
		echo "DBUSER ................ $DBUSER"
		echo "DBNAME ................ $DBNAME"
		echo "DBHOST ................ $DBHOST"
		echo "DBPORT ................ $DBPORT"
		echo
		echo "Database error message:"
		psql -U $DBUSER -d $DBNAME -h $DBHOST -p $DBPORT -c ""
		echo
		echo "Please check your configuration and"
		echo "be sure the database server is up and running."
		exit 1
	fi
	
	# Check log dir and sql dir
	echo -n "Checking sql directory ($SQLDIR) ... "
	if [ ! -e $SQLDIR ]; then
		echo "no"
		echo "!! Missing sql directory ($SQLDIR). Exiting."; exit 1;
	else 
		echo "ok"
	fi
	
	echo -n "Checking log directory ($LOGDIR) ... "
	if [ ! -e $LOGDIR ]; then
        echo "no"
		echo "!! Missing log directory ($LOGDIR)";
		echo "--> Creating log directory ($LOGDIR)";
		mkdir -p $LOGDIR;	
	else
		echo "ok"
	fi

	# reset error flag
	__internal_error=""
}



db_setup() {
	# Create working directory
	echo "--> Creating working directory ($WORKDIR)"
	if [ -e $WORKDIR ] 
	then
		echo "!!  Directory $WORKDIR exists. Removing $WORKDIR."
		rm -rf $WORKDIR
	fi
	mkdir -p $WORKDIR

	# Create log directory
	echo "--> Creating log directory ($LOGDIR)"
	if [ -e $WLOGDIR ]
        then
		echo "!!  Directory $LOGDIR exists. Removing $LOGDIR."
	        rm -rf $LOGDIR
	fi
	mkdir -p $LOGDIR
			

	echo "* Copying necessary files into working directory"
	for i in $SQLFILES
	do
		cp -r "$SQLDIR/$i" $WORKDIR
	done
	
	# Configure SQL template files
	find $WORKDIR -type f -exec sed -i"" -e "s%YourDatasetName%$SCHEMANAME%g" {} \;
	find $WORKDIR -type f -exec sed -i"" -e "s%YourNumberOfChromosomes%$CHRNUM%g" {} \;
	
	# Create schema
	echo "* Inserting new schema into database."
	echo "Please provide the database (${DBNAME}) password for user '${DBUSER}"
	
	case $SCHEMASTRUCT in
		chrpos )  	echo "psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME < $WORKDIR/setup_msqtschema_chrpos.sql;"
					psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME < $WORKDIR/setup_msqtschema_chrpos.sql; # >> $LOGDIR/$LOGFILE 2>&1;
				;;
		posname ) 	echo "psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME < $WORKDIR/setup_msqtschema_posname.sql;"
					psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME < $WORKDIR/setup_msqtschema_posname.sql; # >> $LOGDIR/$LOGFILE 2>&1;
				;;
		* ) echo; echo "!!Invalid dbstruct: ${DBSTRUCT}. Exiting"; exit 1 ;;
	esac
	
	echo 
	echo "Schema configuration finished."
}


#######################################################################################


echo
echo "+-------------------------------------------+"
echo "| This is the MSQT Database Population Tool |"
echo "---------------------------------------------"
echo
read_config_file
test_requirements
test_db
echo
echo
echo "It seems that everything is allright."
echo "We will now proceed the setup"
echo
echo

# Acquire user input
while true; do
	echo
  	echo "The type of data you want to process is"
	echo "  1) chromosome-position structure"
	echo "  2) position-name structure"
	echo
	echo -n "Choose 1 or 2: "
  	read dbstruct
  	case $dbstruct in
    	1 ) SCHEMASTRUCT="chrpos"; break ;;
    	2 ) SCHEMASTRUCT="posname"; break ;;
    	q* ) exit ;;
    	* ) ;;
  	esac
done
echo
case $SCHEMASTRUCT in
	chrpos )	if [ -z "$CHRDIRPREFIX" ]; then
					echo "!! Missing configuration CHRDIRPREFIX information."
					echo "!! Check your config file (${CONFIGFILE})."
					exit 1;
				fi

				while true; do
  					echo -n "How many chromosomes do you have? "
  					read CHRNUM
  					echo $CHRNUM | grep [^0-9] > /dev/null 2>&1;
					if [ "$?" -eq "0" ]; then
						# If the grep found something other than 0-9, then it's not an integer.
						echo "!!Invalid chromosome number: ${CHRNUM}. Wanted a number. Exiting"; 
						
					else
						# The grep found only 0-9, so it's an integer. We can safely do a test on it.
						if [ "$CHRNUM" -le "0" ]; then
							echo "!!Invalid chromosome number: ${CHRNUM}. Should be > 0. Exiting"; 
						else
							# everything is ok
							break;
						fi
					fi
				done;
				break;
				;;
	posname ) break; ;;
	* ) echo "!!Invalid schema structur: '${SCHEMASTRUCT}'. Should be 'chrpos' or 'posname'. Exiting"; exit 1 ;;
esac


# Check user input
case $SCHEMASTRUCT in
	chrpos )  	echo $CHRNUM | grep [^0-9] > /dev/null 2>&1;
				if [ "$?" -eq "0" ]; then
					# If the grep found something other than 0-9, then it's not an integer.
					echo "!!Invalid chromosome number: ${CHRNUM}. Wanted a number. Exiting"; 
					exit 1;
				else
					# The grep found only 0-9, so it's an integer. We can safely do a test on it.
  					if [ "$CHRNUM" -le "0" ]; then
    				echo "!!Invalid chromosome number: ${CHRNUM}. Should be > 0. Exiting"; 
  					fi
			  	fi;
				;;
   	posname ) break; ;;
	* ) echo "!!Invalid schema structur: '${SCHEMASTRUCT}'. Should be 'chrpos' or 'posname'. Exiting"; exit 1 ;;
esac


# Test data directories
case ${SCHEMASTRUCT} in
	chrpos )	for i in `seq 1 $CHRNUM`
				do
					chromdirs="${PATHTODATA}/${CHRDIRPREFIX}${i}"
					
					# Check existance of datadirs
					if [ ! -e $chromdir ]; then
						echo "!! FATAL ERROR: Data directory '${chromdir}' does not exist."
						echo "Please check your configuration. Exiting."
						exit 1
					fi
				
					for dir in $chromdirs
					do
						echo "  ${dir}"
					done
				done
				break ;;
	posname )	# Check existance of datadirs
				if [ ! -e ${PATHTODATA} ]; then
					echo "!! FATAL ERROR: Data directory '${PATHTODATA}' does not exist."
					echo "Please check your configuration. Exiting."
					exit 1
				else
					echo "  ${PATHTODATA}"
				fi
				break ;;
esac

# Prompt user for correctness of variables
echo
echo
echo "Please check the configuration"
echo
echo "DBHOST................. \"${DBHOST}\""
echo "DBPORT................. \"${DBPORT}\""
echo "DBNAME................. \"${DBNAME}\""
echo "SCHEMANAME............. \"${SCHEMANAME}\""
echo "SCHEMASTRUCT........... \"${SCHEMASTRUCT}\""
echo "CHRNUM................. \"${CHRNUM}\""
echo
echo "Please be sure that your postgressql server is up and running"
echo "and that the database '${DBNAME}' exists."
echo
while true; do
  	echo -n "Correct? [y/n] "
	read yn
  	case $yn in
    	y* | Y* ) break ;;
    	n* | N* ) echo; echo "Exiting."; exit 1; break ;;
    	* ) ;;
  	esac
done


# Log variable values
echo "Log file for create_msqt_schema.sh" > $LOGDIR/$LOGFILE
echo "" >> $LOGDIR/$LOGFILE
echo "DBHOST=\"${DBHOST}\"" >> $LOGDIR/$LOGFILE
echo "DBPORT=\"${DBPORT}\"" >> $LOGDIR/$LOGFILE
echo "DBNAME=\"${DBNAME}\"" >> $LOGDIR/$LOGFILE
echo "SCHEMANAME=\"${SCHEMANAME}\"" >> $LOGDIR/$LOGFILE
echo "SCHEMASTRUCT=\"${SCHEMASTRUCT}\"" >> $LOGDIR/$LOGFILE
echo "CHRNUM=\"${CHRNUM}\"" >> $LOGDIR/$LOGFILE


#######################################################################################
#
# 0. Database setup
#
#######################################################################################

echo
while true; do
echo "The schema '${SCHEMANAME}' MUST NOT exist. If it exists, delete it with:"
echo "  delete from public.msqt_config where msqt_schema_name = '${SCHEMANAME}';"
echo "  drop schema ${SCHEMANAME} cascade;"
echo "This issue is subject to change"
echo
  	echo "Do you want to create a new schema? If you say no here,"
	echo -n "be sure that the schema '${SCHEMANAME}' exists. [Y/n] "
	read yn
  	case $yn in
    	y* | Y* ) db_setup; break ;;
    	n* | N* ) break ;;
    	* ) db_setup; break ;;
  	esac
done

#######################################################################################
#
# 1. Process users FASTA files 
# 
#
#######################################################################################
echo
echo
echo "We now process input data in ${SCHEMASTRUCT} format"
echo

# Prompt user for correctness of variables
echo
echo "Checking filenames"

__invalid_filename_err=0;
case ${SCHEMASTRUCT} in
	chrpos )	for i in `seq 1 $CHRNUM`
				do
					chromdirs="${PATHTODATA}/${CHRDIRPREFIX}${i}"
					
					# Check existance of datadirs
					if [ ! -e $chromdir ]; then
						echo "!! FATAL ERROR: Data directory '${chromdir}' does not exist."
						echo "Please check your configuration. Exiting."
						exit 1
					fi
				
					for dir in $chromdirs; do
						# Check filenames, must not contain dashs (-)
						for file in $(find ${dir} -maxdepth 1 -type f  ); do
							perl -e 'use File::Basename;$f=basename $ARGV[0];if($f=~/[- ]/){exit 1;}else{exit 0};' ${file} 
							if [ $? -eq 1 ]; then 
								echo "!! Invalid filename: ${file}"
								__invalid_filename_err=1
							fi 
						done
					done
				done
				break ;;
	posname )	# Check existance of datadirs
				if [ ! -e ${PATHTODATA} ]; then
					echo "!! FATAL ERROR: Data directory '${PATHTODATA}' does not exist."
					echo "Please check your configuration. Exiting."
					exit 1
				else
					# Check filenames, must not contain dashs (-)
					for file in $(find ${PATHTODATA} -type f ); do
						perl -e 'use File::Basename;$f=basename $ARGV[0];if($f=~/[- ]/){exit 1;}else{exit 0};' ${file} 
						if [ $? -eq 1 ]; then 
							echo "!! Invalid filename: ${file}"
							__invalid_filename_err=1
						fi 
					done
				fi
				break ;;
esac


# check result of filename check
if [ ${__invalid_filename_err} -eq "1" ]; then 
	echo "Please rename the invalid files. Aborting."; exit 1;
else
	echo "Filenames are ok, proceeding"
fi 



# Create working directory
echo
echo
echo "--> Creating directory for results ($RESDIR)"
if [ -e $RESDIR ] 
then
	echo "!!  Directory $RESDIR exists. Removing $RESDIR."
    rm -rf $RESDIR
fi
mkdir -p $RESDIR


#######################################################################################
#
# 2. Create a list of ecotypes
#
# get_distinct_etypes.sh creates file ./distinct_ecotype.list
#
#######################################################################################
echo -n "--> Parsing FASTA files and creating list of ecotypes ..."

case $SCHEMASTRUCT in
	chrpos ) 	find ${PATHTODATA}/${CHRDIRPREFIX}* -type f -exec perl -n -e 'print "$1\n" if m/^>(.*)$/;' {} \; > ${RESDIR}/ecot_tmp.list
				break ;;
				
	posname )  	find ${PATHTODATA}/* -type f -exec perl -n -e 'print "$1\n" if m/^>(.*)$/;' {} \; > ${RESDIR}/ecot_tmp.list
				break ;;
				
	* ) echo; echo "!!Invalid dbstruct: ${SCHEMASTRUCT}. Exiting"; exit 1 ;;
esac

cat ${RESDIR}/ecot_tmp.list | sort | uniq > ${RESDIR}/distinct_ecotype.list
rm ${RESDIR}/ecot_tmp.list
echo " done."

# Checking number of distinct ecotypes
noecotypes=`cat ${RESDIR}/distinct_ecotype.list | wc -l`
echo
echo "I found ${noecotypes} distinct ecotypes."

# Suggest user grep oneliner to find weird ecotypes
echo -n "Please check the list of distinct ecotypes. Press return to view list: "
read key
$PAGER ${RESDIR}/distinct_ecotype.list

while true; do
  echo -n "Is the list correct? If not, exit and check your data files! [y/n] "
  read yn
  case $yn in
    y* | Y* ) break ;;
    n* | N* ) echo; echo "Exiting."; exit 1; break ;;
    * ) ;;
  esac
done


echo
echo "There may be missing ecotypes in some FASTA files."
echo "We have to modify your FASTA files to fill in the missing ecotypes."
echo "Please be sure to have a backup of your Data."
echo
while true; do
  echo -n "Ok to adjust the FASTA files? (y/n)"
  read yn
  case $yn in
    y* | Y* ) break ;;
    [nN]* )  echo; echo "!! Aborting. tar is your friend ;)"; exit 1 ;;
    * ) ;;
  esac
done


#######################################################################################
#
# 3. Fill up missing ecotypes with Ns in Fasta files
#
#######################################################################################
echo -n "--> Filling up missing ecotypes with Ns in Fasta files ..."
echo
#echo "find ${PATHTODATA}/${CHRDIRPREFIX}* -type f -exec ./check_for_ecot_completness.pl ${RESDIR}/distinct_ecotype.list target {} \; >> ./log.txt"

case $SCHEMASTRUCT in
	chrpos ) 	find ${PATHTODATA}/${CHRDIRPREFIX}* -type f -exec ./check_for_ecot_completness.pl ${RESDIR}/distinct_ecotype.list target {} \; >> ${LOGDIR}/ecot_completness.log
				break ;;
				
	posname )  	find ${PATHTODATA}/* -type f -exec ./check_for_ecot_completness.pl ${RESDIR}/distinct_ecotype.list target {} \; >> ${LOGDIR}/ecot_completness.log
				break ;;
				
	* ) echo; echo "!!Invalid dbstruct: ${SCHEMASTRUCT}. Exiting"; exit 1 ;;
esac

echo " done."

while true; do
  echo -n "View log of missing ecotypes? (y/n) "
  read yn
  case $yn in
    y* | Y* ) $PAGER ${LOGDIR}/ecot_completness.log; break ;;
    [nN]* )  break ;;
    q* ) exit ;;
    * ) ;;
  esac
done



#######################################################################################
#
# 4. Parsing FASTA files and detect SNPs, Produce target sequence
#    
#    This creates two files: {RESDIR}/target.data ${RESDIR}/snp.data
#
#######################################################################################
echo
echo "!! From now on no more user interaction is necessary"
echo "!! We will mine SNPs and populate the database. This will take a while,"
echo "!! go for a coffee or go home for the weekend"
echo
echo "--> Parsing FASTA files and detect SNPs and extracting target sequence"

__internal_error=""
case $SCHEMASTRUCT in
	chrpos ) 	for i in `seq 1 $CHRNUM`
				do
					chromdir="$PATHTODATA/${CHRDIRPREFIX}${i}"
					FILES=`ls ${chromdir}/*`
					for file in $FILES
					do
						current_frag=`basename ${file}`
						echo -n "  - Processing fragment ${current_frag} on chromosome $i ..."
						
						./get_allelels_batch3.pl chrpos $file $i ${RESDIR}/target.data ${RESDIR}/snp.data
						if [ $? -eq 0 ] ; then echo "done"
						else __internal_error=1;
							echo "failed !!"
						fi
						
					done
				done
				break ;;
				
	posname )  	FILES=`ls ${PATHTODATA}/*`
				for file in $FILES
				do
					current_frag=`basename ${file}`
					echo -n "  - Processing fragment ${current_frag} ..."
					
					./get_allelels_batch3.pl posname $file ${RESDIR}/target.data ${RESDIR}/snp.data
					if [ $? -eq 0 ] ; then echo "done"
            		else __internal_error=1;
	          			echo "failed !!"; 
					fi

				
				done
				break ;;
				
	* ) echo; echo "!!Invalid dbstruct: ${SCHEMASTRUCT}. Exiting"; exit 1 ;;
esac

#Check status

echo
if [ ${__internal_error} ]; then
	echo "While parsing the FASTA files some errors occured."
	exit;
else
	echo "Parsing FASTA files done."
fi

# Reset error flag
__internal_error=""
					     



#######################################################################################
#
# 6. Copy data into MQT databse; target sequence and snps
#
#######################################################################################
# snps
#psql ${DBNAME} -U ${DBUSER} -W ${DBPASS} -c "\COPY ${SCHEMANAME}.snps(chromosome, position, ecotype, replacement, frag_id, allele_freq) FROM  '${RESDIR}/snp.data'"

#target seq
# psql ${DBNAME} -U ${DBUSER} -W ${DBPASS} -c "\COPY ${SCHEMANAME}.target_sequences(chromosome, position, ref_base, frag_id) FROM  '${RESDIR}/target.data'"

echo "Copying data into database. Please provide the database (${DBNAME}) password for user '${DBUSER}' (3 times):"
case $SCHEMASTRUCT in
	chrpos ) 	psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "\COPY ${SCHEMANAME}.snps(chromosome, position, ecotype, replacement, frag_id, allele_freq) FROM '${RESDIR}/snp.data'"
				psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "\COPY ${SCHEMANAME}.target_sequences(chromosome, position, ref_base, frag_id) FROM '${RESDIR}/target.data'"
				psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "INSERT INTO ${SCHEMANAME}.ecotypes (ecotype) SELECT DISTINCT ${SCHEMANAME}.snps.ecotype FROM ${SCHEMANAME}.snps;"
				break ;;
				
	posname )  	#psql -q -h $DBHOST -p $DBPORT -U $DBUSER -W -d $DBNAME -c
				psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "\COPY ${SCHEMANAME}.snps(position_name, ecotype, replacement, frag_id, allele_freq) FROM '${RESDIR}/snp.data'"
				psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "\COPY ${SCHEMANAME}.target_sequences(position_name, ref_base, frag_id) FROM '${RESDIR}/target.data'"
				psql -q -h $DBHOST -p $DBPORT -U $DBUSER -d $DBNAME -c "INSERT INTO ${SCHEMANAME}.ecotypes (ecotype) SELECT DISTINCT ${SCHEMANAME}.snps.ecotype FROM ${SCHEMANAME}.snps;"
			break ;;
esac
#######################################################################################
#
# 7. Compute MSQT/SNIPED database
#
#######################################################################################
echo
echo "Database Setup complete. Have a nice day."
