#!/usr/bin/perl
#
# Extract SNPs from FASTA alignments
# written 2006 by Joffrey Fitz (fitzj@informatik.uni-tuebingen.de)
#

use strict;
use File::Basename;

my ($mode, $file, $chrom, $target_res_file, $snp_res_file);
$mode = shift(@ARGV);

if($mode eq "chrpos") {
	($file, $chrom, $target_res_file, $snp_res_file) = @ARGV;
}
elsif($mode eq "posname"){
	($file, $target_res_file, $snp_res_file) = @ARGV;
}
else {
	die "Unknown mode '$mode'. Try 'posname' or 'chrpos'.\n";
}

my ($fragname, $fragid);
my @ecot;	# The ecotype array
my @m;		# The sequence matrix

# Parse command line
if( ($mode =~ /^posname$/) && ($file ne "") && ($target_res_file ne "") && ($snp_res_file ne "") ) {
	$fragname=basename($file);	
}
elsif( ($mode =~ /^chrpos$/) && ($file ne "") && ($chrom ne "") && ($target_res_file ne "") && ($snp_res_file ne "") ) {
	$fragid=basename($file);
}
else {
	print "USAGE: $0 chrpos <fasta-file> <chromosome> <target.data res file> <snp.data res file>\n";
	print "USAGE: $0 posname <fasta-file> <target.data res file> <snp.data res file>\n";
	exit(1);
}

# Read FASTA file and generate nucleotide matrix
open(FH, "$file") or die $!;
while(<FH>) {
	chomp;
	if(/^>(.*)$/) { @ecot = (@ecot, $1); }
	else { #@seq = (@seq, $_); }
		my @tmp = split //, $_;
    	push @m, [ @tmp ];
	}
}

# Get count of Ecotypes
my $cnt_ecot = scalar(@ecot);

# Extract target sequence
# TBD

# Finding index of 'target'
my($i, $match_idx);
for ($i = 0; $i < @ecot; $i++) {
    if ($ecot[$i] =~ /target/) {
        $match_idx = $i;    # save the index
        last;
	}
}

# Determine length of target sequence
my $l_target = length(join //, @m[$match_idx]);
for my $aref ( @m[$match_idx] ) {
        my $len = join(/,/, @$aref);
        $l_target = length($len)-1;
}

# Export target sequence:
# chromosome, position, replacement, frag_id
#print STDERR "*  Exporting target sequence for frgagment $file into $target_res_file\n";
open( TARGET_OUT, ">>$target_res_file") or die $!;

for my $sp ( 0 .. $l_target) {
	if($mode eq "chrpos") {
		my $position = $sp+$fragid;
		print TARGET_OUT "$chrom\t$position\t$m[$match_idx][$sp]\t$fragid\n";
	}
	elsif($mode eq "posname"){
		my $position_name = "$fragname-$sp";
		print TARGET_OUT "$position_name\t$m[$match_idx][$sp]\t$fragname\n";
	}
	else {
		die "Unknown mode '$mode'. Exiting.\n";
	}
}
close TARGET_OUT;

# Analyze the matrix

open( SNP_OUT, ">>$snp_res_file") or die $!;
#print STDERR "*  Exporting SNP data for fragment $file into $snp_res_file\n";

# Init hashes for nucleotide count and frequencies
my %nuc_cnt = ('A',0,'C',0,'G',0,'T',0,'B',0,'D',0,'H',0,'V',0,'R',0,'Y',0,'K',0,'M',0,'S',0,'W',0,'N',0,'-',0);
my %nuc_freq = %nuc_cnt;

for my $sp ( 0 .. $l_target) {
	my $col="";
	for my $z ( 0 .. $#m) {
		$col .= $m[$z][$sp];
	}
	# Pick out homogenous columns
	if ($col =~ /^A+$|^T+$|^G+$|^C+$|^-+$|^B+$|^D+$|^H+$|^V+$|^R+$|^Y+$|^K+$|^M+$|^S+$|^W+$/) { 
		#print "  Only AATGC in $sp: '$col'\n" if($DEBUG); 
	}
	# Pick out columns containing only Ns and one other base like (GGGNGGG) 
	elsif ($col =~ /^[AN]+$|^[TN]+$|^[GN]+$|^[CN]+$|^[-N]+$|^[BN]+$|^[DN]+$|^[HN]+$|^[VN]+$|^[RN]+$|^[YN]+$|^[KN]+$|^[MN]+$|^[SN]+$|^[WN]+$/) { 
		#print "  Homogenous + N in $sp: '$col'\n" if($DEBUG); 
	}
	else {
		# SNP candidate!
		#print "SNP at $sp: $col\n" if($DEBUG);
		
		# Count the occurance of each nucletide
		foreach my $key (sort keys %nuc_cnt) {
			$nuc_cnt{$key} = grep(/$key/, split(//,$col));
		}

		# Compute allele frequency for each nocletide
		foreach my $key (sort keys %nuc_freq) {
			$nuc_freq{$key} = 1 / $cnt_ecot * $nuc_cnt{$key} if($nuc_cnt{$key}!=0);
		}

		# Print output:
		# chromosome, position, ecotype, replacement, frag_id, allele_freq
		for my $_z ( 0 .. $#m ) {
			
			if($mode eq "chrpos") {
				my $position = $sp+$fragid;
				print SNP_OUT "$chrom\t$position\t$ecot[$_z]\t$m[$_z][$sp]\t$fragid\t";
			}
			elsif($mode eq "posname"){
				my $position = $sp;
				print SNP_OUT "$fragname-$position\t$ecot[$_z]\t$m[$_z][$sp]\t$fragname\t";
			}
			else {
				die "Unknown mode '$mode'. Exiting.\n";
			}
			foreach my $key (sort keys %nuc_freq) {
				if( $m[$_z][$sp] eq "$key") { print SNP_OUT sprintf("%.2f", $nuc_freq{$key}) . "\n"; }
			}
		}	
	}
}
close SNO_OUT;
exit 0;

