#!/usr/bin/perl 
#
# snipe4snps.pl
#
# Version 0.2
#
# Mine best SNPs for each fragment
#
# Written 2006 by Joffrey Fitz
#
# This script is part of the MSQT project by
# Norman Warthmann and Joffrey Fitz
# 
# Synopsis: Do not use this directly.
# This script is called by snipe4snps.sh
#
#
use strict;

use DBI;
use DBD::Pg;
use POSIX;
use distinct_snps_chrpos;
use parse_conf;

$| = 1;
my $DEBUG = 0;

# Database stuff
my $username = "msqt";

# Check command line arguments
my $schema = $ARGV[0];
if ($schema eq "") {
	print STDERR "ERROR\n";
	print STDERR "No msqt schema given.\n";
	exit 1;
}


# Read and check configuration
parsefile("../../config/msqt.conf");
checkconf();


print STDERR <<EOC;
Welcome to snipe4snps!

Program was started with following parameters:
  - Host:....................... $host
  - Database:................... $dbname
  - Schema:..................... $schema
  - Div threshold:.............. $div_threshold
  - Right neighb. threshold:.... $right_neigbh_threshold
  - Left neighb. threshold:..... $left_neigbh_threshold
  
  
EOC

# Open database connection
my $dbh = DBI->connect("dbi:Pg:dbname=$dbname;host=$host;port=$port;".
	"", "$username", "$password",
        {AutoCommit => 1} ) || db_connect_error();

my $out;
my $sth;

# get count of entries in table schema.sniped
# if count > 0, user must empty the table!
my $query_entries = <<EOQ;
	SELECT count(*) FROM $schema.sniped;
EOQ

$sth = $dbh->prepare( $query_entries );
$sth->execute();
my $entries = $sth->fetchrow;
$sth->finish;

if ($entries > 0) {
	print STDERR "ERROR:";
	print STDERR "Tabel $schema.sniped is not empty. It contains $entries tuples.\n";
	print STDERR "Please delete old tuples with\n";
	print STDERR "  psql -h $host -U $username -p $port -c \"DELETE FROM $schema.sniped;\"\n\n";
	exit 1;
}


# get number of chromosomes. this is stored in msqt_config
my $query_chromosomes = <<EOQ;
	SELECT chromosomes FROM public.msqt_config WHERE msqt_schema_name = '$schema';
EOQ

$sth = $dbh->prepare( $query_chromosomes );
$sth->execute();
my $chromosomes = $sth->fetchrow;
$sth->finish;

if ($chromosomes > 0) {
	print STDERR  "Found $chromosomes chromosomes for schema '$schema' on database '$dbname'\n";
}
else {
	print STDERR "ERROR:";
	print STDERR "No information about chromosomes found in public.msqt_config.\n";
	print SDTERR "Please check your configuration!\n";
	exit 1;
}

#
# loop through all available chromosomes starting at 1
#
for(my $chrom=1; $chrom <=$chromosomes; $chrom++) 
{
	print "Gathering fragment ids for chromosome $chrom\n";
	
	#  get all frag_ids on current chromosome
	my $query_fragid = <<EOQ;
		SELECT DISTINCT frag_id FROM $schema.snps WHERE chromosome='$chrom' order by frag_id;
EOQ
	
	$sth = $dbh->prepare( $query_fragid );
	$sth->execute();
	
	my @fragids = qw//;
	while(my @row = $sth->fetchrow_array){
		push(@fragids, $row[0]);
	}
	$sth->finish;

	#
	# Loop through all distinct frag_ids found on current chromosome
	#
	foreach my $frag_id (@fragids) {
		# clear hash
		my %hash;
			
		print "\nProcessing frag_id $frag_id\n\n";
	
	
		# get all positions for current fragid
		my $query_position = <<EOQ;
			SELECT DISTINCT position from $schema.snps 
			where frag_id='$frag_id' 
				and replacement != 'B' 
				and replacement != 'D'
				and replacement != 'H'
				and replacement != 'V'
				and replacement != 'R'
				and replacement != 'Y'
				and replacement != 'K'
				and replacement != 'M'
				and replacement != 'S'
				and replacement != 'W'
				and replacement != 'N'   
			AND chromosome = $chrom 
			ORDER BY position ASC; 
EOQ
		
		$sth = $dbh->prepare( $query_position ) || die;
		$sth->execute() || die;
		
		# There must be at least 1 position!
		if($sth->rows < 1) {
			print "***$frag_id: Only $sth->rows position found. Skipping\n";
			next;
		}
		
		my @positions = qw//;
		while(my @row = $sth->fetchrow_array){
			push(@positions, $row[0]);
		}
		$sth->finish;
		

		$dbh->{ pg_server_prepare => 1 };
		my $query_allele_freq = <<EOQ;
			SELECT DISTINCT replacement, allele_freq, 
				CAST( abs( (allele_freq - 0.5) ) AS NUMERIC (3,2)) AS norm_allele_freq 
			from $schema.snps 
			WHERE position = ? 
				and replacement != 'B' 
				and replacement != 'D'
				and replacement != 'H'
				and replacement != 'V'
				and replacement != 'R'
				and replacement != 'Y'
				and replacement != 'K'
				and replacement != 'M'
				and replacement != 'S'
				and replacement != 'W'
				and replacement != 'N' 
				AND chromosome = $chrom 
			ORDER BY norm_allele_freq ASC LIMIT 2;
EOQ

		$sth = $dbh->prepare($query_allele_freq) || die; 
		
		print STDERR "Checking diff-value criterion: <$div_threshold\n";
		
print "Query: ",$query_allele_freq,"\n" if($DEBUG);

		foreach my $i (@positions) {
			
			$sth->execute($i) || die;
			my $query_rows = $sth->rows;	
		
			# There must be at least 2 alleles/ecotypes!
			if($query_rows < 2) { 
				print "***$i: Only $query_rows allele found. Skipping\n"; 
				next; 
			}
			
			#alleles with frequence near to 0.5
			my $first_allele; 
			my $second_allele;
			
			my @allele_freqs = qw//;
			my @alleles = qw//;
			while(my @row = $sth->fetchrow_array){
				#my $med = 0;
				my $med = abs($row[1] - 0.5);
				push(@alleles, $row[0]);
				push(@allele_freqs, $row[1]);	
			}
			# compute sum(allele_freq1, allele_freq2)
			my $sum = $allele_freqs[0]+$allele_freqs[1];
			my $div = abs($allele_freqs[0]-$allele_freqs[1]);
			
			# chrom, $frag_id, position, allele1, allele1_freq, allele2, allele2_freq, freq_sum, freq_div
			#print STDERR "$chrom\t$frag_id\t$i\t$alleles[0]\t$allele_freqs[0]\t$alleles[1]\t$allele_freqs[1]\t$sum\t$div\n";
			
			# cut off threshold after div
			############
			print STDERR "$chrom\t$frag_id\t$i\t$alleles[0]\t$allele_freqs[0]\t$alleles[1]\t$allele_freqs[1]\t$sum";
			if($div_threshold > $div ) {
				print STDERR "\t$sum\t$div\t[diff-value ok]\n";
				my $info = $chrom.":".$frag_id.":".$i.":".$alleles[0].":".$allele_freqs[0].":".$alleles[1].":".$allele_freqs[1].":".$sum;
				$hash{$info} = $div;
			}
			else {
				print STDERR "\t$sum\t$div\t[** diff-value:$div SKIPPING]\n";
			}
		}
		$sth->finish; 
		
		
		# ADF stuff
		############
		
		
		
		my $ecot_sth = $dbh->prepare(
			"select ecotype from $schema.snps where frag_id= ? and chromosome= ? and replacement= ? and allele_freq= ? and position= ?;"
		);
		
		my $insert_sth = $dbh->prepare("
		insert into $schema.sniped (
		chrom, fragid, position,summary, 
		allele1, allele_freq1,
		allele2, allele_freq2,
		freq_sum, freq_diff,
		neighborhood_left_len, neighborhood_right_len,
		ecot1, ecot2, adf
		)
		VALUES (
		?,?,?,?,?,?,?,?,?,?,?,
		?,
		?,
		?, ?
		)
		");
		
		# loop all remaining snps
		print STDERR "Checking neighborhood criterion: left $left_neigbh_threshold, right:$right_neigbh_threshold\n";
		
		foreach my $key (sort { $hash{$a} <=> $hash{$b} } (keys %hash)) { # sort hash ascending
			my $div = $hash{$key};
			my @values = split(/:/,$key);
			my $chrom   = $values[0];
			my $frag_id = $values[1];
			my $position = $values[2];
			my $allele1 = $values[3];
			my $allele2 = $values[5];
			my $allele_freq1 = $values[4];
			my $allele_freq2 = $values[6];
			my $sum = $values[7];
			
		
			my $snp_list = new distinct_snps_chrpos();
			my %ethash;
		
			# get first ecotype list
			$ecot_sth->execute($frag_id, $chrom, $allele1, $allele_freq1, $position);
			my @ecotlist1 = undef;
			while(my @ecot_row = $ecot_sth->fetchrow_array){
				$ethash{$ecot_row[0]} = 0;	
			}
			$ecot_sth->finish;
			
			# get first ecotype list
			$ecot_sth->execute($frag_id, $chrom, $allele2, $allele_freq2, $position);
			my @ecotlist2 = undef;
			while(my @ecot_row = $ecot_sth->fetchrow_array) {
				$ethash{$ecot_row[0]} = 1;	
			}
			$ecot_sth->finish;
		
			$ethash{"target"} = 2;
			
			$snp_list->find_frag($chrom, $position, $dbh, $schema, %ethash);
			$snp_list->genaissance_html($dbh);
		
			my $adf_raw = $snp_list->{gen_out};
			my @adf_lines = split(/\n/, $adf_raw);
			
			# pick out the line containing information about the position
			my $adf;
			foreach my $i (@adf_lines) {
				if($i =~ m/$position/ ) {
					$adf = $i;
				}
			}
			
			# get rid of position trailing sequence	
			#$adf =~ s/\d*://;
		
			#
			# Check neighborhood
			#
			my ($rnhl, $lnhl);
		
			# check for left neighborhood
			if( $adf =~ m/^.*\}(.*)\[/ ) {
				$lnhl = length($1);
				#print "neighborhood left:" , $1 , " Length:", $lnlh, "\n";
			}
			elsif( $adf =~ m/^(.*)\[/ ) {
				$lnhl = length($1);
				#print "neighborhood left:" , $1 , " Length:", $lnlh, "\n";
			}
			# check for right neighborhood
			if(	$adf =~ m/^.*\](.*?)\{.*/ ) {
				$rnhl = length($1);
				#print "neighborhood right:" , $1 , " Length:", $rnhl, "\n";
			}
			elsif(	$adf =~ m/^.*\](.*)$/ ) {
				$rnhl = length($1);
				#print "neighborhood right:" , $1 , " Length:", $rnhl, "\n";
			}
			
			#
			# check here neighborhood constraints
			#
			if( ($rnhl < $right_neigbh_threshold) or ($lnhl < $left_neigbh_threshold) ) {
				print STDERR "** Skipping chr:$chrom, frag:$frag_id, pos:$position due to rnhl:$rnhl, lnhl:$lnhl, diff-value:$div\n";
				#next SNP;
			}
			else {
				print STDERR "\n==> Choosen SNP:\n";
				print STDERR "==> $chrom\t$frag_id\t$position\t$allele1\t$allele_freq1\t$allele2\t$allele_freq2\t$sum\t$div\n";
				print STDERR "==> diff-value:$div\n";
				my $summary = "|<--$lnhl-[$allele1/$allele2]-$rnhl-->|";
				print STDERR "==> summary: $summary\n";
				
				#print STDERR "\nSkipping all other candidates.\n\n";
				
				#
				# print output
				#
				#chrom, $frag_id, position, allele1, allele1_freq, allele2, allele2_freq, freq_sum, freq_div, left_nhl, right_nlh
		# 		print "$chrom\t$frag_id\t$position\t$allele1\t$allele_freq1\t$allele2\t$allele_freq2\t";
		# 		print "$sum\t$div\t";
		# 		print "$lnhl\t$rnhl\t";
		# 		# print ecotype lists
				my ($etlist1, $etlist2);
				foreach my $key (keys %ethash) {
					if ($ethash{$key} == 0) {
						$etlist1 .= $key . ",";
					}
					elsif ($ethash{$key} == 1) {
						$etlist2 .= $key . ",";
					}
				}
				chop $etlist1; chop $etlist2;
		# 		# snp sequence in adf style
		# 		print "$etlist1\t$etlist2\t";
		# 		print "$adf";
			
				#
				# execute SQL inserts
				#
				$insert_sth->execute(
					$chrom, $frag_id, $position, $summary, $allele1, $allele_freq1, 
					$allele2, $allele_freq2, $sum, $div, 
					$lnhl, $rnhl, $etlist1, $etlist2, $adf
				);
		
				# this is for skipping al other candidates
				#last;
			}
		}
	
	} # end of frag_id loop
} # end of chromosome loop


exit(1);


